/*
LCD drive

	RA5:O:LCD E
	RC0:O:LCD RS
	RC1:0:LCD RW(R/W is only VSS)
	RA0:O:LCD D4
   	RA1:0:LCD D5
	RA2:O:LCD D6
	RA3:O:LCD D7
	DB0 to DB3 are must be open

	RA4:1:BUZZER

*/

#include <htc.h>
#include "main.h"
#include "lcd.h"


static bit LCD_RS	@ ((unsigned)&PORTC*8+0);	// Register select (RC0)
static bit LCD_EN	@ ((unsigned)&PORTA*8+5);	// Enable (RA5)
static bit LCD_RW	@ ((unsigned)&PORTC*8+1);	// ONLY VSS (RC1)



// write a byte to the LCD in 4 bit mode
void lcd_data_byteset(unsigned char c){
    LCD_RW = 0;
	PORTA = (PORTA & 0b11110000) |  (c & 0b00001111);		//Only low 4bit DATA
	LCD_STROBE;
}

//LCD data set
void lcd_data_set(unsigned char c){
    LCD_RW = 0;
	PORTA = (PORTA & 0b11110000) | ( (c >>4 ) & 0b00001111 ) ;	//Hi SIDE DATA
	LCD_STROBE;
	PORTA = (PORTA & 0b11110000) |  (c & 0b00001111 );		//low SIDE DATA
	LCD_STROBE;
}

//LCD write
void
lcd_write(unsigned char c)
{
    LCD_RW = 0;
	LCD_RS = 0;
	PORTA = (PORTA & 0b11110000) | ( (c >>4 ) & 0b00001111 ) ;	//Hi SIDE DATA
	LCD_STROBE;
	PORTA = (PORTA & 0b11110000) |  (c & 0b00001111 );		//low SIDE DATA
	LCD_STROBE;
//	__delay_us(40);	
	delay_us(4);	
}

//Clear and home the LCD
void
lcd_clear(void)
{
	lcd_write(0x1);
	lcd_loccnt=0;
//	__delay_ms(2); //PICC Pro
	delay_ms(2); //PICC Pro
}

//write a string of chars to the LCD
void
lcd_puts(const char * s)
{
	LCD_RS = 1;	// write characters
	while(*s)
		lcd_putch(*s++);
}

//write one character to the LCD and auto increment location counter*/
void
lcd_putch(unsigned char c)
{
	LCD_RS = 1;	// write characters
	lcd_data_set(c);
	delay_us(4);
	lcd_loccnt++;
}

//write to CG RAM 
void
lcd_cgwrite(unsigned char c)
{
	LCD_RS = 1;	// write characters
	lcd_data_set(c);
//	__delay_us(40);
	delay_us(4);
}

//Go to the specified position

void
lcd_goto(unsigned char pos)
{
	lcd_write(0x80+pos);
	lcd_loccnt=pos;
}
	
//initialise the LCD - put into 4 bit mode
void
lcd_init(void)
{
	LCD_RS = 0;				// write control bytes
	delay_ms(15);			// power on delay 
	lcd_data_byteset(0x03);	// attention!
	delay_ms(5);			//wait
	LCD_STROBE;				//strobe again
	delay_us(10);		//wait
	LCD_STROBE;				//strobe again
	delay_ms(5);			//wait
	lcd_data_byteset(0x02) ;// set 4 bit mode
	delay_us(4);			//wait
	lcd_write(0x28);		// 4 bit mode, 1/16 duty, 5x8 font
	lcd_write(0x08);		// display off
	lcd_write(0x0F);		// display on, blink curson on
	lcd_write(0x06);		// entry mode
	lcd_loccnt=0;
}

//Newline processing
void lcd_newline(unsigned char mode){
	unsigned char temp;
	switch (mode){
		case 1://the following are deleted
			if (lcd_loccnt <= LCD_LineLength1){
				lcd_space_full(LCD_LineLength1-lcd_loccnt);
				return;
			}else if (lcd_loccnt <= LCD_LineLength2) {
				lcd_space_full(LCD_LineLength2-lcd_loccnt);
				return;
			}else return;

		default://toggle the line
			if (lcd_loccnt <= LCD_LineLength1) lcd_goto(LCD_LineNumber2);
			else if (lcd_loccnt <= LCD_LineLength2) lcd_goto(LCD_LineNumber1);
	}
}	
void lcd_space_full(unsigned char count){//spacing
	while(count--){
		lcd_putch(' ');//space character write
	};
}		
		


// 10Sec ^C} (Secł͂ȂA)----------------------
// 8MHz
void delay_us(int usec){
	while(usec !=0){
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
#ifdef	CPU1938
		asm("NOP");
		asm("NOP");
#endif

		usec--;
	}
}	

// mSec ^C}----------------------
void delay_ms(int msec){
	int i;
	for(i=0; i< msec; i++)
		delay_us(100);
}

/*******************************************************************************
*    lcd_puth - 1oCgl16iLCDɕ\鏈                     *
*      c : ϊPoCg̐lw肷                                    *
********************************************************************************/
void lcd_puth(unsigned char c)
{
     const char HEX_Data[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'} ;

     lcd_putch(HEX_Data[ (c >> 4) ]) ;
     lcd_putch(HEX_Data[ c & 0x0F ]) ;
}




void lcd_time(unsigned long t_data , int line){
	char buffer;
	int i	;

	buffer = (t_data % 360000)/36000 ;		// \̐l ASCII ɂ
	for(unsigned int i=1; i< 4; i++){
		if (buffer == 0){
			lcd_puts(" ");
		}else{
			lcd_putch(LCD_FONT[buffer][line * 4 + i]);
		}
	}
	lcd_puts(" ");

	buffer = (t_data % 36000)/3600 ;		// ꎞ̐l ASCII ɂ
	for(unsigned int i=0; i< 4; i++){
		lcd_putch(LCD_FONT[buffer][line * 4 + i]);
	}

	switch(line){
		case 0:
			lcd_puts(" ");
			lcd_putch(0x01);
			lcd_puts(" ");
		break;
		case 1:
			lcd_puts("   ");
		break;
		default:
			lcd_puts(" ");
			lcd_putch(0x04);
			lcd_puts(" ");
		break;
	}

	buffer = (t_data % 3600)/600 ;			// \̐l ASCII ɂ
	for(unsigned int i=0; i< 4; i++){
		lcd_putch(LCD_FONT[buffer][line * 4 + i]);
	}
	lcd_puts(" ");
	buffer = (t_data % 600)/60 ;			// ꕪ̐l ASCII ɂ
//	buffer = (t_data % 60)/5;		// \b̐l ASCII ɂ
//	buffer = (t_data % 10) ;		// b̐l ASCII ɂ
	for(unsigned int i=0; i< 4; i++){
		lcd_putch(LCD_FONT[buffer][line * 4 + i]);
	}

}


void lcd_time_4l(unsigned long t_data){

	unsigned char barline,barcha,chaline ;
	char buffer;
	barline = (unsigned char)(t_data % 60);
	barcha = barline / 5 ;
	chaline = barline % 5 ;

	lcd_write(0x44);				//CG RAM[h 1ڂɃZbg
	for (unsigned char i=4; i< 8; i++){
		switch(chaline){
			case 0:
				lcd_cgwrite(0x00);
			break;
			case 1:
				lcd_cgwrite(0x10);				
			break;
			case 2:
				lcd_cgwrite(0x18);				
			break;
			case 3:
				lcd_cgwrite(0x1c);				
			break;
			case 4:
				lcd_cgwrite(0x1e);				
			break;
			default:
				lcd_cgwrite(0x1f);				
			break;
		}
	}

	lcd_goto(LCD_LineNumber4);	//4sڕ\

	buffer = (t_data % 360000)/36000 + '0';		// \̐l ASCII ɂ
	if (buffer == '0'){
		lcd_puts(" ");
	}else{
		lcd_putch(buffer);
	}
	buffer = (t_data % 36000)/3600 + '0';		// ꎞ̐l ASCII ɂ
	lcd_putch(buffer);
	lcd_puts(":");
	buffer = (t_data % 3600)/600 + '0';			// \̐l ASCII ɂ
	lcd_putch(buffer);
	buffer = (t_data % 600)/60 + '0';			// ꕪ̐l ASCII ɂ
	lcd_putch(buffer);
	lcd_puts(":");
	buffer = (t_data % 60)/10 + '0';		// \b̐l ASCII ɂ
	lcd_putch(buffer);
	buffer = (t_data % 10) + '0';		// b̐l ASCII ɂ
	lcd_putch(buffer);

	for (unsigned char j=0; j< 12; j++){

		if (barcha < j){
			lcd_puts(" ");
		}else if (barcha == j){
			lcd_putch(0x00);

		}else{
			lcd_putch(0x01);	
		}
	}
}
