/*
 *	LCD interface header file
 *	See lcd.c for more info
 */

//LCD location parameter
//LCD drive for PC2004 20 columns 4 lines 

#include "main.h"
#define	LCD_LineNumber1	0x00

#ifdef	LM2560
	#define	LCD_LineNumber2	0x20
	#define	LCD_LineNumber3	0x40
	#define	LCD_LineNumber4	0x60
#else
	#define	LCD_LineNumber2	0x40
	#define	LCD_LineNumber3	0x14
	#define	LCD_LineNumber4	0x54
#endif


#define	LCD_LineLength1	0x14	//LCD 20x4


//LCD line length
#define	LCD_LineLength2	(LCD_LineLength1 + LCD_LineNumber2)

//LCD location parameter
//LCD drive for SC1602BS 16 columns 2 lines 
#define	LCD_CursorBlink	0x0f
#define	LCD_CursorOn	0x0e
#define	LCD_CursorOff	0x0c

/*	Set the cursor position */
#define	lcd_cursor(x)	lcd_write(((x)&0x7F)|0x80)

//LCD STROBE (general specification is more than 500ns)
#define	LCD_STROBE	(((LCD_EN = 1),(LCD_EN=0))
//extern  char latAO;
//#define	LCD_STROBE ((latAO = latAO | 0b00100000),(PORTA = latAO),(latAO = latAO & 0b11011111),(PORTA = latAO))	

/* write a byte to the LCD in 4 bit mode */
extern void lcd_write(unsigned char);

/* Clear and home the LCD */
extern void lcd_clear(void);

/* write a string of characters to the LCD */
extern void lcd_puts(const char * s);

/* Go to the specified position */
extern void lcd_goto(unsigned char pos);

/* intialize the LCD - call before anything else */
extern void lcd_init(void);
extern void lcd_putch(unsigned char);

/*	Now LCD position memory */
unsigned char lcd_loccnt=0;

/* CG RAM write data set */
extern void lcd_cgwrite(unsigned char);

//LCD new line 
extern void lcd_newline(unsigned char);

//LCD space full
void lcd_space_full(unsigned char);


extern void delay_us(int usec);
extern void delay_ms(int msec);

extern void lcd_puth(unsigned char c);

extern void lcd_time(unsigned long t_data , int line);
extern void lcd_time_4l(unsigned long t_data);

const char LCD_FONT[][12] = {
		{	//0
			0x05,0x04,0x04,0x05,
			0x03,0x20,0x20,0x03,
			0x06,0x01,0x01,0x06
		},
		{	//1
			0x20,0x01,0x03,0x20,
			0x20,0x20,0x03,0x20,
			0x20,0x01,0x03,0x01
		},
		{	//2
			0x05,0x04,0x04,0x05,
			0x01,0x02,0x02,0x04,
			0x03,0x01,0x01,0x01
		},
		{	//3
			0x02,0x04,0x04,0x05,
			0x20,0x02,0x02,0x03,
			0x02,0x01,0x01,0x06
		},
		{	//4
			0x20,0x01,0x03,0x20,
			0x03,0x01,0x03,0x01,
			0x20,0x20,0x03,0x20

		},
		{	//5
			0x03,0x04,0x04,0x04,
			0x06,0x02,0x02,0x01,
			0x02,0x01,0x01,0x04
		},
		{	//6
			0x05,0x04,0x20,0x20,
			0x03,0x02,0x02,0x01,
			0x06,0x01,0x01,0x04
		},
		{	//7
			0x03,0x04,0x04,0x03,
			0x20,0x20,0x01,0x04,
			0x20,0x20,0x03,0x20
		},
		{	//8
			0x05,0x04,0x04,0x05,
			0x03,0x02,0x02,0x03,
			0x06,0x01,0x01,0x06
		},
		{	//9
			0x05,0x04,0x04,0x05,
			0x04,0x02,0x02,0x03,
			0x20,0x20,0x01,0x04
		}
};





const char lcd_cg[][8] = {	

		{	//0x00
			0b00000,
			0b00000,
			0b00000,
			0b00000,
			0b00000,
			0b00000,
			0b00000,
			0b00000
		},
		{	//0x01
			0b00000,
			0b00000,
			0b00000,
			0b00000,
			0b11111,
			0b11111,
			0b11111,
			0b11111
		},
		{	//0x02
			0b00000,
			0b00000,
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b00000,
			0b00000
		},
		{	//0x03 
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b11111
		},
		{	//0x04
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b00000,
			0b00000,
			0b00000,
			0b00000
		},
		{	//0x05
			0b00000,
			0b00000,
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b11111
		},

		{	//0x06
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b11111,
			0b00000,
			0b00000
		},

		{	//0x07
			0b11111,
			0b11111,
			0b00000,
			0b00000,
			0b00000,
			0b00000,
			0b11111,
			0b11111
		}
};